@@ -17,6 +17,7 @@ from tabulate import tabulate
 
 from liualgotrader.analytics.analysis import load_trades_by_batch_id
 from liualgotrader.common import config, market_data, trading_data
+from liualgotrader.common.data_loader import DataLoader
 from liualgotrader.common.database import create_db_connection
 from liualgotrader.common.decorators import timeit
 from liualgotrader.common.tlog import tlog
@@ -27,6 +28,7 @@ from liualgotrader.models.trending_tickers import TrendingTickers
 from liualgotrader.scanners.base import Scanner
 from liualgotrader.scanners.momentum import Momentum
 from liualgotrader.strategies.base import Strategy, StrategyType
+from liualgotrader.trading.alpaca import AlpacaTrader
 
 
 def get_batch_list():
@@ -37,7 +39,7 @@ def get_batch_list():
         print(
             tabulate(
                 data,
-                headers=["build", "batch_id", "strategy", "env", "start time"],
+                headers=["build", "batch_id", "strategy", "start time"],
             )
         )
 
@@ -63,14 +65,14 @@ starting
 
 def show_usage():
     print(
-        f"usage: {sys.argv[0]} --batch-list OR [--strict] [--symbol=SYMBOL] [--debug=SYMBOL] [--duration=<minutes>] <batch-id>\n"
+        f"usage:\n{sys.argv[0]} batch --batch-list OR [--strict] [--symbol=SYMBOL] [--debug=SYMBOL] [--duration=<minutes>] <batch-id>",
+        "\nOR"
+        f"\n{sys.argv[0]} from <start_date> [--scanners=<scanner-name,>] [--strats=<strategy-name,>] [--to=<end_date> DEFAULT is today] [--scale=day(DEFAULT)|minute",
     )
     msg = """
-    'backter' application re-runs a past trading session, with new or modified
-     strategies specified in tradeplan.toml. 'backter' application looks for 
-     tradeplan.toml in current directory. The 'backter' application expects a 
-     batch-id (UDID) as input. Using the --batch-list option you 
-     can see a list of recent sessions to choose from. 
+    'backter' application re-runs a past trading session, or test strategies on past data. 
+    Re-run past sessions to test modified strategies or back-test new strategies on past data before running  
+    on paper, or live accounts. To learn more, read LiuAlgoTrader online documentation.
     """
     print(msg)
     print("options:")
@@ -82,11 +84,15 @@ def show_usage():
         "--duration\tRun back-test for number of <minutes>, bypass batch-id run duration"
     )
     print(
-        "--debug\tWrite verbose debug information for symbol SYMBOL during back-testing"
+        "--debug\t\tWrite verbose debug information for symbol SYMBOL during back-testing"
     )
     print(
         "--strict\tRun back-test session only on same symbols traded in the original batch"
     )
+    print(
+        "--to\t\tdate string in the format YYYY-MM-DD, if not provided current day is selected"
+    )
+    print("--scale\t\ttime-scale for loading past data for back-test-ing")
 
 
 def show_version(filename: str, version: str) -> None:
@@ -100,6 +106,7 @@ async def create_strategies(
     ref_run_id: Optional[int],
     uid: str,
     start: datetime,
+    data_loader: DataLoader,
     bypass_strategy_duration: bool = False,
 ) -> None:
     strategy_types = []
@@ -137,11 +144,11 @@ async def create_strategies(
                 traceback.print_exc()
                 exit(0)
 
+    config.env = "BACKTEST"
     for strategy_tuple in strategy_types:
         strategy_type = strategy_tuple[0]
         strategy_details = strategy_tuple[1]
-        config.env = "BACKTEST"
-        tlog(f"initializing {strategy_type.name}")
+        tlog(f"initializing {strategy_type.__name__}")
 
         if "schedule" not in strategy_details:
             print("duration:", duration)
@@ -161,7 +168,10 @@ async def create_strategies(
                 schedule["duration"] = duration.total_seconds() // 60
 
         s = strategy_type(
-            batch_id=uid, ref_run_id=ref_run_id, **strategy_details
+            batch_id=uid,
+            ref_run_id=ref_run_id,
+            data_loader=data_loader,
+            **strategy_details,
         )
         await s.create()
         trading_data.strategies.append(s)
@@ -169,7 +179,7 @@ async def create_strategies(
 
 @timeit
 async def backtest_symbol(
-    data_api: tradeapi,
+    data_loader: DataLoader,
     portfolio_value: float,
     symbol: str,
     start: datetime,
@@ -200,53 +210,29 @@ async def backtest_symbol(
     if debug_symbol:
         print("--> using DEBUG mode")
 
-    re_try = 3
+    symbol_data = pd.DataFrame(
+        data_loader[symbol][start_time : start_time + duration]  # type: ignore
+    )
+    add_daily_vwap(
+        symbol_data,
+        debug=debug_symbol,
+    )
+    print(
+        f"loaded {len(symbol_data)} agg data points({start_time}-{start_time + duration})"
+    )
 
-    while re_try > 0:
-        # load historical data
-        try:
-            symbol_data = data_api.polygon.historic_agg_v2(
-                symbol,
-                1,
-                "minute",
-                _from=str(start_time - timedelta(days=8)),
-                to=str(start_time + timedelta(days=1)),
-                limit=10000,
-            ).df
-        except HTTPError as e:
-            tlog(f"Received HTTP error {e} for {symbol}")
-            return
-
-        if len(symbol_data) < 100:
-            tlog(f"not enough data-points  for {symbol}")
-            return
-
-        add_daily_vwap(
-            symbol_data,
-            debug=debug_symbol,
-        )
-        market_data.minute_history[symbol] = symbol_data
-        print(
-            f"loaded {len(market_data.minute_history[symbol].index)} agg data points"
-        )
-        try:
-            minute_index = symbol_data["close"].index.get_loc(
-                start_time, method="nearest"
-            )
-            break
-        except (Exception, ValueError) as e:
-            print(f"[EXCEPTION] {e} - trying to reload-data. ")
-            re_try -= 1
-
-    if re_try <= 0:
-        return
+    minute_index = symbol_data["close"].index.get_loc(
+        start_time, method="nearest"
+    )
 
     position: int = 0
     new_now = symbol_data.index[minute_index]
     print(f"start time with data {new_now}")
     price = 0.0
     last_run_id = None
+
     # start_time + duration
+    rejected: Dict[str, List] = {}
     while (
         new_now < config.market_close
         and minute_index < symbol_data.index.size - 1
@@ -262,6 +248,8 @@ async def backtest_symbol(
                 print(
                     f"Execute strategy {strategy.name} on {symbol} at {new_now}"
                 )
+            if symbol in rejected.get(strategy.name, []):
+                continue
 
             try:
                 do, what = await strategy.run(
@@ -275,6 +263,7 @@ async def backtest_symbol(
                     backtesting=True,
                 )
             except Exception as e:
+                traceback.print_exc()
                 tlog(
                     f"[ERROR] exception {e} on symbol {symbol} @ {strategy.name}"
                 )
@@ -324,29 +313,33 @@ async def backtest_symbol(
                         symbol, price, int(float(what["qty"]))
                     )
                     break
+            elif what.get("reject", False):
+                if strategy.name in rejected:
+                    rejected[strategy.name].append(symbol)
+                else:
+                    rejected[strategy.name] = [symbol]
+
             last_run_id = strategy.algo_run.run_id
 
         minute_index += 1
         new_now = symbol_data.index[minute_index]
 
-    if position:
-        if (
-            trading_data.last_used_strategy[symbol].type
-            == StrategyType.DAY_TRADE
-        ):
-            tlog(f"[{new_now}]{symbol} liquidate {position} at {price}")
-            db_trade = NewTrade(
-                algo_run_id=last_run_id,  # type: ignore
-                symbol=symbol,
-                qty=int(position) if int(position) > 0 else -int(position),
-                operation="sell" if position > 0 else "buy",
-                price=price,
-                indicators={"liquidate": 1},
-            )
-            await db_trade.save(
-                config.db_conn_pool,
-                str(symbol_data.index[minute_index - 1]),
-            )
+    if position and (
+        trading_data.last_used_strategy[symbol].type == StrategyType.DAY_TRADE
+    ):
+        tlog(f"[{new_now}]{symbol} liquidate {position} at {price}")
+        db_trade = NewTrade(
+            algo_run_id=last_run_id,  # type: ignore
+            symbol=symbol,
+            qty=int(position) if int(position) > 0 else -int(position),
+            operation="sell" if position > 0 else "buy",
+            price=price,
+            indicators={"liquidate": 1},
+        )
+        await db_trade.save(
+            config.db_conn_pool,
+            str(symbol_data.index[minute_index - 1]),
+        )
 
 
 def backtest(
@@ -357,11 +350,7 @@ def backtest(
     specific_symbols: List[str] = None,
     bypass_duration: int = None,
 ) -> str:
-    data_api: tradeapi = tradeapi.REST(
-        base_url=config.prod_base_url,
-        key_id=config.prod_api_key_id,
-        secret_key=config.prod_api_secret,
-    )
+    data_loader = DataLoader()
     portfolio_value: float = (
         100000.0 if not config.portfolio_value else config.portfolio_value
     )
@@ -414,19 +403,20 @@ def backtest(
                 ref_run_id,
                 uid,
                 start,
+                DataLoader(),
                 bypass_duration is not None,
             )
 
             for symbol_and_start_time in symbols_and_start_time:
                 symbol = symbol_and_start_time[0]
                 await backtest_symbol(
-                    data_api=data_api,
+                    data_loader=data_loader,
                     portfolio_value=portfolio_value,
                     symbol=symbol,
                     start=start,
                     duration=duration,
                     scanner_start_time=symbol_and_start_time[1],
-                    debug_symbol=True if symbol in debug_symbols else False,
+                    debug_symbol=symbol in debug_symbols,
                 )
 
     @timeit
@@ -442,16 +432,14 @@ def backtest(
                 start=min(starts),
                 duration=timedelta(
                     minutes=max(
-                        [
-                            w["duration"]
-                            for w in [
-                                item
-                                for sublist in [
-                                    conf_dict["strategies"][s]["schedule"]  # type: ignore
-                                    for s in conf_dict["strategies"]  # type: ignore
-                                ]
-                                for item in sublist
+                        w["duration"]
+                        for w in [
+                            item
+                            for sublist in [
+                                conf_dict["strategies"][s]["schedule"]  # type: ignore
+                                for s in conf_dict["strategies"]  # type: ignore
                             ]
+                            for item in sublist
                         ]
                     )
                     if not bypass_duration
@@ -484,15 +472,9 @@ class BackTestDay:
     def __init__(self, conf_dict: Dict):
         self.uid = str(uuid.uuid4())
 
-        self.data_api: tradeapi = tradeapi.REST(
-            base_url=config.prod_base_url,
-            key_id=config.prod_api_key_id,
-            secret_key=config.prod_api_secret,
-        )
-
         self.conf_dict = conf_dict
         config.portfolio_value = self.conf_dict.get("portfolio_value", None)
-        self.minute_history: Dict[str, pd.DataFrame] = {}
+        self.data_loader = DataLoader()
         self.scanners: List[Scanner] = []
 
     async def create(self, day: date) -> str:
@@ -521,8 +503,8 @@ class BackTestDay:
                         "target_strategy_name", None
                     )
                     scanner_object = Momentum(
-                        provider=scanner_details["provider"],
-                        data_api=self.data_api,
+                        data_loader=self.data_loader,
+                        trading_api=AlpacaTrader(),
                         min_last_dv=scanner_details["min_last_dv"],
                         min_share_price=scanner_details["min_share_price"],
                         max_share_price=scanner_details["max_share_price"],
@@ -566,13 +548,13 @@ class BackTestDay:
                     scanner_details.pop("filename")
                     if "recurrence" not in scanner_details:
                         scanner_object = custom_scanner(
-                            data_api=self.data_api,
+                            data_loader=self.data_loader,
                             **scanner_details,
                         )
                     else:
                         recurrence = scanner_details.pop("recurrence")
                         scanner_object = custom_scanner(
-                            data_api=self.data_api,
+                            data_loader=self.data_loader,
                             recurrence=timedelta(minutes=recurrence),
                             **scanner_details,
                         )
@@ -590,6 +572,7 @@ class BackTestDay:
             None,
             self.uid,
             day.replace(hour=9, minute=30, second=0, microsecond=0),
+            DataLoader(),
         )
 
         self.now = pd.Timestamp(self.start)
@@ -603,132 +586,119 @@ class BackTestDay:
 
     async def next_minute(self) -> Tuple[bool, List[Optional[str]]]:
         rc_msg: List[Optional[str]] = []
-        if self.now < self.end:
-            for i in range(0, len(self.scanners)):
-                if self.now == self.start or (
-                    self.scanners[i].recurrence is not None
-                    and self.scanners[i].recurrence.total_seconds() > 0  # type: ignore
-                    and int((self.now - self.start).total_seconds() // 60)  # type: ignore
-                    % int(self.scanners[i].recurrence.total_seconds() // 60)  # type: ignore
-                    == 0
-                ):
-                    new_symbols = await self.scanners[i].run(self.now)
-                    if new_symbols:
-                        really_new = [
-                            x for x in new_symbols if x not in self.symbols
-                        ]
-                        if len(really_new) > 0:
-                            print(
-                                f"Loading data for {len(really_new)} symbols: {really_new}"
-                            )
-                            rc_msg.append(
-                                f"Loaded data for {len(really_new)} symbols: {really_new}"
-                            )
-                            self.minute_history = {
-                                **self.minute_history,
-                                **(
-                                    market_data.get_historical_data_from_poylgon_for_symbols(
-                                        self.data_api,
-                                        really_new,
-                                        self.start - timedelta(days=7),
-                                        self.start + timedelta(days=1),
-                                    )
-                                ),
-                            }
-                            self.symbols += really_new
-                            print(f"loaded data for {len(really_new)} stocks")
-
-            for symbol in self.symbols:
-                try:
-                    for strategy in trading_data.strategies:
-
-                        try:
-                            minute_index = self.minute_history[symbol][
-                                "close"
-                            ].index.get_loc(self.now, method="nearest")
-                        except Exception as e:
-                            print(f"[Exception] {self.now} {symbol} {e}")
-                            print(self.minute_history[symbol]["close"][-100:])
-                            continue
-
-                        price = self.minute_history[symbol]["close"][
-                            minute_index
-                        ]
-
-                        if symbol not in trading_data.positions:
-                            trading_data.positions[symbol] = 0
-
-                        do, what = await strategy.run(
-                            symbol,
-                            True,
-                            int(trading_data.positions[symbol]),
-                            self.minute_history[symbol][: minute_index + 1],
-                            self.now,
-                            self.portfolio_value,
-                            debug=False,  # type: ignore
-                            backtesting=True,
-                        )
-                        if do:
-                            if (
-                                what["side"] == "buy"
-                                and float(what["qty"]) > 0
-                                or what["side"] == "sell"
-                                and float(what["qty"]) < 0
-                            ):
-                                trading_data.positions[symbol] += int(
-                                    float(what["qty"])
-                                )
-                                trading_data.buy_time[
-                                    symbol
-                                ] = self.now.replace(second=0, microsecond=0)
-                            else:
-                                trading_data.positions[symbol] -= int(
-                                    float(what["qty"])
-                                )
-
-                            trading_data.last_used_strategy[symbol] = strategy
-
-                            rc_msg.append(
-                                f"[{self.now}][{strategy.name}] {what['side']} {what['qty']} of {symbol} @ {price}"
-                            )
-                            db_trade = NewTrade(
-                                algo_run_id=strategy.algo_run.run_id,
-                                symbol=symbol,
-                                qty=int(float(what["qty"])),
-                                operation=what["side"],
-                                price=price,
-                                indicators=trading_data.buy_indicators[symbol]
-                                if what["side"] == "buy"
-                                else trading_data.sell_indicators[symbol],
-                            )
-
-                            await db_trade.save(
-                                config.db_conn_pool,
-                                str(self.now.to_pydatetime()),
-                                trading_data.stop_prices[symbol],
-                                trading_data.target_prices[symbol],
-                            )
-
-                            if what["side"] == "buy":
-                                await strategy.buy_callback(
-                                    symbol, price, int(float(what["qty"]))
-                                )
-                                break
-                            elif what["side"] == "sell":
-                                await strategy.sell_callback(
-                                    symbol, price, int(float(what["qty"]))
-                                )
-                                break
-                except Exception as e:
-                    print(f"[Exception] {self.now} {symbol} {e}")
-                    traceback.print_exc()
-
-            self.now += timedelta(minutes=1)
-
-            return True, rc_msg
-        else:
+        if self.now >= self.end:
             return False, []
 
+        for i in range(len(self.scanners)):
+            if self.now == self.start or (
+                self.scanners[i].recurrence is not None
+                and self.scanners[i].recurrence.total_seconds() > 0  # type: ignore
+                and int((self.now - self.start).total_seconds() // 60)  # type: ignore
+                % int(self.scanners[i].recurrence.total_seconds() // 60)  # type: ignore
+                == 0
+            ):
+                new_symbols = await self.scanners[i].run(self.now)
+                if new_symbols:
+                    really_new = [
+                        x for x in new_symbols if x not in self.symbols
+                    ]
+                    if really_new:
+                        print(
+                            f"Loading data for {len(really_new)} symbols: {really_new}"
+                        )
+                        rc_msg.append(
+                            f"Loaded data for {len(really_new)} symbols: {really_new}"
+                        )
+                        self.symbols += really_new
+                        print(f"loaded data for {len(really_new)} stocks")
+
+        for symbol in self.symbols:
+            try:
+                for strategy in trading_data.strategies:
+
+                    try:
+                        minute_index = self.data_loader[symbol][
+                            "close"
+                        ].index.get_loc(self.now, method="nearest")
+                    except Exception as e:
+                        print(f"[Exception] {self.now} {symbol} {e}")
+                        print(self.data_loader[symbol]["close"][-100:])
+                        continue
+
+                    price = self.data_loader[symbol]["close"][minute_index]
+
+                    if symbol not in trading_data.positions:
+                        trading_data.positions[symbol] = 0
+
+                    do, what = await strategy.run(
+                        symbol,
+                        True,
+                        int(trading_data.positions[symbol]),
+                        self.data_loader[symbol][: minute_index + 1],
+                        self.now,
+                        self.portfolio_value,
+                        debug=False,  # type: ignore
+                        backtesting=True,
+                    )
+                    if do:
+                        if (
+                            what["side"] == "buy"
+                            and float(what["qty"]) > 0
+                            or what["side"] == "sell"
+                            and float(what["qty"]) < 0
+                        ):
+                            trading_data.positions[symbol] += int(
+                                float(what["qty"])
+                            )
+                            trading_data.buy_time[symbol] = self.now.replace(
+                                second=0, microsecond=0
+                            )
+                        else:
+                            trading_data.positions[symbol] -= int(
+                                float(what["qty"])
+                            )
+
+                        trading_data.last_used_strategy[symbol] = strategy
+
+                        rc_msg.append(
+                            f"[{self.now}][{strategy.name}] {what['side']} {what['qty']} of {symbol} @ {price}"
+                        )
+                        db_trade = NewTrade(
+                            algo_run_id=strategy.algo_run.run_id,
+                            symbol=symbol,
+                            qty=int(float(what["qty"])),
+                            operation=what["side"],
+                            price=price,
+                            indicators=trading_data.buy_indicators[symbol]
+                            if what["side"] == "buy"
+                            else trading_data.sell_indicators[symbol],
+                        )
+
+                        await db_trade.save(
+                            config.db_conn_pool,
+                            str(self.now.to_pydatetime()),
+                            trading_data.stop_prices[symbol],
+                            trading_data.target_prices[symbol],
+                        )
+
+                        if what["side"] == "buy":
+                            await strategy.buy_callback(
+                                symbol, price, int(float(what["qty"]))
+                            )
+                            break
+                        elif what["side"] == "sell":
+                            await strategy.sell_callback(
+                                symbol, price, int(float(what["qty"]))
+                            )
+                            break
+            except Exception as e:
+                print(f"[Exception] {self.now} {symbol} {e}")
+                traceback.print_exc()
+
+        self.now += timedelta(minutes=1)
+
+        return True, rc_msg
+
     async def liquidate(self):
         for symbol in trading_data.positions:
             if (
@@ -737,10 +707,10 @@ class BackTestDay:
                 == StrategyType.DAY_TRADE
             ):
                 position = trading_data.positions[symbol]
-                minute_index = self.minute_history[symbol][
-                    "close"
-                ].index.get_loc(self.now, method="nearest")
-                price = self.minute_history[symbol]["close"][minute_index]
+                minute_index = self.data_loader[symbol]["close"].index.get_loc(
+                    self.now, method="nearest"
+                )
+                price = self.data_loader[symbol]["close"][minute_index]
                 tlog(f"[{self.end}]{symbol} liquidate {position} at {price}")
                 db_trade = NewTrade(
                     algo_run_id=trading_data.last_used_strategy[symbol].algo_run.run_id,  # type: ignore
