@@ -9,11 +9,12 @@ from liualgotrader.common.tlog import tlog
 
 async def create_db_connection(dsn: str = None) -> None:
     config.db_conn_pool = await asyncpg.create_pool(
-        dsn=dsn if dsn else config.dsn,
+        dsn=dsn or config.dsn,
         min_size=2,
-        max_size=10,
-        command_timeout=30,
+        max_size=50,
+        command_timeout=60,
     )
+
     tlog("db connection pool initialized")
 
 
@@ -28,4 +29,9 @@ async def fetch_as_dataframe(query: str, *args) -> pd.DataFrame:
         stmt = await con.prepare(query)
         columns = [a.name for a in stmt.get_attributes()]
         data = await stmt.fetch(*args)
-        return pd.DataFrame(data=data, columns=columns)
+
+        return (
+            pd.DataFrame(data=data, columns=columns)
+            if data and len(data) > 0
+            else pd.DataFrame()
+        )
