@@ -16,10 +16,10 @@ latest_cost_basis: Dict[str, float] = {}
 latest_scalp_basis: Dict[str, float] = {}
 sell_indicators: Dict[str, Dict] = {}
 buy_indicators: Dict[str, Dict] = {}
-positions: Dict[str, float] = {}
+positions: Dict[str, int] = {}
 target_prices: Dict[str, float] = {}
 stop_prices: Dict[str, float] = {}
-partial_fills: Dict[str, float] = {}
+partial_fills: Dict[str, int] = {}
 symbol_resistance: Dict[str, float] = {}
 voi: Dict[str, List[float]] = {}
 voi_ask: Dict[str, Tuple[float, float, datetime]] = {}
