@@ -15,7 +15,10 @@ class AlgoRun:
         self.batch_id = batch_id
 
     async def save(
-        self, pool: Pool = None, env: str = None, ref_algo_run_id: int = None
+        self,
+        pool: Pool = None,
+        env: str = "unknown",
+        ref_algo_run_id: int = None,
     ) -> None:
         if not pool:
             pool = config.db_conn_pool
@@ -32,7 +35,7 @@ class AlgoRun:
                     self.run_id = await con.fetchval(
                         q,
                         self.strategy_name,
-                        env if env else config.env,
+                        env or "unknown",
                         config.build_label,
                         json.dumps(
                             {
@@ -41,6 +44,7 @@ class AlgoRun:
                         ),
                         self.batch_id,
                     )
+
                 else:
                     q = """
                         INSERT INTO algo_run (algo_name, algo_env, build_number, parameters, batch_id, ref_algo_run)
@@ -50,7 +54,7 @@ class AlgoRun:
                     self.run_id = await con.fetchval(
                         q,
                         self.strategy_name,
-                        env if env else config.env,
+                        env or "unknown",
                         config.build_label,
                         json.dumps(
                             {
@@ -107,7 +111,7 @@ class AlgoRun:
             async with con.transaction():
                 rows = await con.fetch(
                     """
-                        SELECT build_number, batch_id, algo_name, algo_env, date_trunc('minute', min(start_time)) as start
+                        SELECT build_number, batch_id, algo_name, date_trunc('minute', min(start_time)) as start
                         FROM algo_run
                         WHERE start_time > $2
                         GROUP BY batch_id, algo_name, algo_env, build_number
